# -*- coding: utf-8; mode: cython -*-

cdef extern from "finitediff_c.h":
     cdef int finitediff_calculate_weights(double *, int, double *, int, int, double)
     cdef int finitediff_calc_and_apply_fd(double *, int, int, int, int, double *, double *, int, double)
     cdef int finitediff_interpolate_by_finite_diff(double * out, int, int, int, int, int, int, int, double *, int, double *, int, double *)



from .. import adapted_grid
from ..util import locate_discontinuity, pool_discontinuity_approx
from ._common import g3


def test_locate_discontinuity__pool_discontinuity_approx():
    for snr in [False, True]:
        loc_res = locate_discontinuity(*adapted_grid(0, 10, g3, grid_additions=(16,)*8, snr=snr),
                                       consider=5)
        avg, s = pool_discontinuity_approx(loc_res)
        if snr:
            assert abs(avg - 5) < 0.03
        else:
            assert abs(avg - 5) > 1  # this is a negative test, i.e. not strict!



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, division, print_function)

import math
import numpy as np


def sigm(x, m, o, n):
    return m*(x-o)*((m*(x-o))**n + 1)**-(1./n)


def g(x):
    s = sigm(x, 20, 1, 4)
    s2 = sigm(x, 5, 1, 4)
    g.nelem_call += x.size
    g.nfev += 1
    return 10*(1-s2**2)*np.exp(20*s)/np.exp(20*x)+0.1*(x-3)**2

g.nelem_call = 0
g.nfev = 0


def g2(x):
    return g(2*x) + g(x)

g3 = np.vectorize(lambda t: (
    math.exp(-t) + (.2 if round(t*1234567) % 3 == 0 else 0)
) if t < 5 else 0.5*math.exp(5-t))  # noisy function
